import json
import os

def load_custom_dataset(json_file_path):
    """加载自定义数据集，区分 train 和 test 数据"""
    train_data = []
    test_data = []

    # 读取 JSON 文件
    with open(json_file_path, "r", encoding="utf-8") as f:
        data = json.load(f)

    # 遍历数据，按照 'split' 将数据分配到 train 或 test
    for item in data:
        split = item["metadata"].get("split", "train")  # 默认是 'train'
        if split == "train":
            train_data.append(item)
        elif split == "test":
            test_data.append(item)
        else:
            # 如果有其他的 split 类型，可以根据需要处理
            print(f"Unknown split: {split}")

    return train_data, test_data


json_file_path = "/home/hukun/code/trimodal_code/craft_mol/data/Mol-Instructions/data/Molecule-oriented_Instructions/molecular_description_generation.json"
train_data, test_data = load_custom_dataset(json_file_path)

# 打印加载的 train 和 test 数据的前 2 条样本
print("Train Data (First 2 samples):")
print(train_data[:2])

print("\nTest Data (First 2 samples):")
print(test_data[:2])



import glob


import torch
import pubchempy
import pandas as pd
from tqdm import tqdm
from concurrent.futures import ThreadPoolExecutor, as_completed
import time
from torch_geometric.data import InMemoryDataset
import selfies as sf
class PubChemDataset(InMemoryDataset):
    def __init__(self, path):
        super(PubChemDataset, self).__init__()
        self.data, self.slices = torch.load(path)
    
    def __getitem__(self, idx):
        return self.get(idx)

# 加载数据集
data = PubChemDataset('/home/hukun/code/Tmm-llama/data/PubChem324kV2/PubChem324kV2/pretrain.pt')

# 函数：获取iUPAC名称，添加超时处理
def get_iupac_name(smiles, timeout=60):
    try:
        compounds = pubchempy.get_compounds(smiles, namespace='smiles', timeout=timeout)
        if compounds:
            return compounds[0].iupac_name
        return None
    except Exception as e:
        print(f"Error fetching IUPAC name for {smiles}: {e}")
        return None

# 并行获取iUPAC名称
def fetch_data_for_index(i):
    sample = data[i]
    smiles = sample["smiles"]
    cid = sample["cid"]
    text = sample["text"]
    # 获取 iupac name
    iupac_name = get_iupac_name(smiles)
    
    if iupac_name is not None:
        return [cid, smiles, text, iupac_name, None]  # 返回数据
    else:
        return [None, None, None, None, i]  # 返回失败的索引



def get_iupac_index_range(index_set:set):
    
    csv_file = f'./data/pubchem324k/pubchem_data_unfinished2.csv'
    df_columns = ["CID", "SMILES", "text", "iupac name"]
    df = pd.DataFrame(columns=df_columns)
    df.to_csv(csv_file, index=False)  # 写入表头

    # 准备存储数据的列表
    completed_indexes = set()  # 用于存储已经完成的索引
    unfinished_indexes = []  # 用于记录未完成的索引


    # 使用 ThreadPoolExecutor 来并行处理数据
    with ThreadPoolExecutor(max_workers=16) as executor:
        futures = []
        
        # 遍历数据集，为每个索引提交任务
        for i in sorted(index_set):
            futures.append(executor.submit(fetch_data_for_index, i))
        
        # 处理完成的任务
        for future in tqdm(as_completed(futures), desc="Processing", total=len(futures), unit="sample"):
            cid, smiles, text, iupac_name,failed_idx = future.result()
            
            if iupac_name is not None:
                # 保存有效数据
                temp_df = pd.DataFrame([[cid, smiles, text, iupac_name]], columns=df_columns)
                temp_df.to_csv(csv_file, mode='a', header=False, index=False)
                completed_indexes.add(failed_idx)  # 记录成功爬取的索引
            else:
                unfinished_indexes.append(failed_idx)  # 记录失败的索引

    # 保存未完成数据的索引到一个文件
    unfinished_file = f'./data/pubchem324k/unhandle_2.txt'
    with open(unfinished_file, 'w') as f:
        for idx in unfinished_indexes:
            f.write(f"{idx}\n")


    print("数据已成功保存为 CSV 文件！")
    print(f"未完成的数据索引已保存到 {unfinished_file}！")

def get_unfinished_set():
    # 定义文件名模式
    file_pattern = "./data/pubchem324k/unhandle_1.txt"

    # 初始化一个空集合
    combined_set = set()

    # 遍历所有匹配的文件
    for file_name in glob.glob(file_pattern):
        with open(file_name, 'r') as file:
            # 读取每一行并转换为整数，添加到集合中
            for line in file:
                try:
                    combined_set.add(int(line.strip()))
                except ValueError:
                    print(f"跳过非数字行: {line.strip()} (文件: {file_name})")

    # 打印合并后的集合
    print(len(combined_set))
    return combined_set


if __name__ == "__main__":
    index_set = get_unfinished_set()
    get_iupac_index_range(index_set)
    print("done")



import re


"""
iupac tokenizer
"""
def smiles_atom_tokenizer (smi):
    pattern =  "(\[[^\]]+]|Br?|Cl?|N|O|S|P|F|I|b|c|n|o|s|p|\(|\)|\.|=|#|-|\+|\\\\|\/|:|~|@|\?|>|\*|\$|\%[0-9]{2}|[0-9])"
    regex = re.compile(pattern)
    tokens = [token for token in regex.findall(smi)]
    return tokens

def duplicates(lst, item):
    return [i for i, x in enumerate(lst) if x == item]

def split_tokens(tokens_to_be_splitted, all_tokens):
    if ''.join(tokens_to_be_splitted) in all_tokens:
        indexes = duplicates(all_tokens, ''.join(tokens_to_be_splitted))
        for k, idx in enumerate(indexes):
            all_tokens[idx] = tokens_to_be_splitted[0]
            for i in range(1, len(tokens_to_be_splitted)):
                all_tokens.insert(idx+i, tokens_to_be_splitted[i])
            if k < len(indexes) - 1:
                indexes[k+1:] = list(map(lambda x: x+(len(tokens_to_be_splitted) - 1), indexes[k+1:]))
def add_symbol(pat):
    new_patt = pat[1:].split(',')
    ls_del = []
    if new_patt[0][0] == '0':
        ls_del.append(new_patt[0][0])
        new_patt[0] = new_patt[0][1:]

    while int(new_patt[0]) > int(new_patt[1]):
        ls_del.append(new_patt[0][0])
        new_patt[0] = new_patt[0][1:]
    result = ['.', ''.join(ls_del), '^', new_patt[0], ',', new_patt[1]]
    return result

def check_correct_tokenize(iupac,tokens):
    if ''.join(tokens).replace('^', '') == iupac:
        return True
    else:
        return False

def iupac_tokenizer(iupac):

    pattern = "\.\d+,\d+|[1-9]{2}\-[a-z]\]|[0-9]\-[a-z]\]|[1-9]{2}[a-z]|[1-9]{2}'[a-z]|[0-9]'[a-z]|[0-9][a-z]|\([0-9]\+\)|\([0-9]\-\)|" + \
              "[1-9]{2}|[0-9]|-|\s|\(|\)|S|R|E|Z|N|C|O|'|\"|;|λ|H|,|\.|\[[a-z]{2}\]|\[[a-z]\]|\[|\]|"

    alcane = "methane|methanoyl|methan|ethane|ethanoyl|ethan|propanoyl|propane|propan|propa|butane|butanoyl|butan|buta|pentane|" + \
             "pentanoyl|pentan|hexane|hexanoyl|hexan|heptane|heptanoyl|heptan|octane|octanoyl|octan|nonane|nonanoyl|" + \
             "nonan|decane|decanoyl|decan|icosane|icosan|cosane|cosan|contane|contan|"

    pristavka_name = "hydroxide|hydroxyl|hydroxy|hydrate|hydro|cyclo|spiro|iso|"
    pristavka_digit = "mono|un|bis|bi|dicta|di|tetraza|tetraz|tetra|tetr|pentaza|pentaz|penta|hexaza|" + \
                      "hexa|heptaza|hepta|octaza|octa|nonaza|nona|decaza|deca|kis|"

    prefix_alcane = "methylidene|methyl|ethyl|isopropyl|propyl|isobutyl|sec-butyl|tert-butyl|butyl|pentyl|hexyl|heptyl|octyl|"
    carbon = "meth|eth|prop|but|pent|hex|hept|oct|non|dec|icosa|icos|cosa|cos|icon|conta|cont|con|heni|hene|hen|hecta|hect|"

    prefix_all = "benzhydryl|benzoxaza|benzoxaz|benzoxa|benzox|benzo|benzyl|benz|phenacyl|phenanthro|phenyl|phenoxaza|phenoxaz|phenoxy|phenox|phenol|pheno|phen|acetyl|aceto|acet|" + \
                 "peroxy|oxido|oxino|oxalo|oxolo|oxocyclo|oxol|oxoc|oxon|oxo|oxy|pyrido|pyrimido|imidazo|naphtho|stiboryl|stibolo|"

    prefix_isotope = "protio|deuterio|tritio|"
    suffix_isotope = "protide|"
    prefix_galogen = "fluoro|fluoranyl|fluoridoyl|fluorido|chloro|chloranyl|chloridoyl|chlorido|bromo|bromanyl|bromidoyl|bromido|iodo|iodanyl|iodidoyl|iodanuidyl|iodido|"
    suffix_galogen = "fluoride|chloride|chloridic|perchloric|bromide|iodide|iodane|hypoiodous|hypochlorous|"
    prefix_сhalcogen = "phosphonato|phosphoroso|phosphonia|phosphoryl|phosphanyl|arsono|arsanyl|stiba|"
    suffix_сhalcogen = "phosphanium|phosphate|phosphite|phosphane|phosphanide|phosphonamidic|phosphonous|phosphinous|phosphinite|phosphono|arsonic|stibane|"
    prefix_metal = "alumanyl|gallanyl|stannyl|plumbyl|"
    suffix_metal = "chromium|stannane|gallane|alumane|aluminane|aluminan|"
    prefix_non_metal = "tellanyl|germanyl|germyl|"
    suffix_non_metal = "germane|germa|"

    prefix_sulfur = "sulfanylidene|sulfinamoyl|sulfonimidoyl|sulfinimidoyl|sulfamoyl|sulfonyl|sulfanyl|sulfinyl|sulfinato|sulfenato|" + \
                    "sulfonato|sulfonio|sulfino|sulfono|sulfido|"
    suffix_sulfur = "sulfonamide|sulfinamide|sulfonamido|sulfonic|sulfamic|sulfinic|sulfuric|thial|thione|thiol|" + \
                    "sulfonate|sulfite|sulfate|sulfide|sulfinate|sulfanium|sulfamate|sulfane|sulfo|"

    prefix_nitrogen = "hydrazono|hydrazino|nitroso|nitrous|nitro|formamido|amino|amido|imino|imido|anilino|anilin|thiocyanato|cyanato|cyano|azido|azanidyl|azanyl|" + \
                      "azanide|azanida|azonia|azonio|amidino|nitramido|diazo|"
    suffix_nitrogen = "ammonium|hydrazide|hydrazine|hydrazin|amine|imine|oxamide|nitramide|formamide|cyanamide|amide|imide|amidine|isocyanide|azanium|" + \
                      "thiocyanate|cyanate|cyanic|cyanatidoyl|cyanide|nitrile|nitrite|hydrazonate|"

    suffix_carbon = "carbonitrile|carboxamide|carbamimidothioate|carbodithioate|carbohydrazonate|carbonimidoyl|carboximidoyl|" + \
                    "carbamimidoyl|carbamimidate|carbamimid|carbaldehyde|carbamate|carbothioyl|carboximidothioate|carbonate|" + \
                    "carboximidamide|carboximidate|carbamic|carbonochloridate|carbothialdehyde|carbothioate|carbothioic|carbono|carbon|carbo|" + \
                    "formate|formic|"
    prefix_carbon = "carboxylate|carboxylato|carboxylic|carboxy|halocarbonyl|carbamoyl|carbonyl|carbamo|thioformyl|formyl|"

    silicon = "silanide|silane|silole|silanyl|silyloxy|silylo|silyl|sila|"
    boron = "boranyl|boranuide|boronamidic|boranuida|boranide|borinic|borate|borane|boran|borono|boron|bora|"
    selenium = "selanyl|seleno|"

    suffix_all = "ane|ano|an|ene|enoxy|eno|en|yne|yn|yl|peroxol|peroxo|" + \
                 "terephthalate|terephthalic|phthalic|phthalate|oxide|oate|ol|oic|ic|al|ate|ium|one|"

    carbon_trivial = "naphthalen|naphthal|inden|adamant|fluoren|thiourea|urea|anthracen|acenaphthylen|" + \
                     "carbohydrazide|annulen|aniline|acetaldehyde|benzaldehyde|formaldehyde|phthalaldehyde|acephenanthrylen|" + \
                     "phenanthren|chrysen|carbanid|chloroform|fulleren|cumen|formonitril|fluoranthen|terephthalaldehyde|azulen|picen|" + \
                     "pyren|pleiaden|coronen|tetracen|pentacen|perylen|pentalen|heptalen|cuban|hexacen|oxanthren|ovalen|aceanthrylen|"

    heterocycles = "indolizin|arsindol|indol|furan|furo|piperazin|pyrrolidin|pyrrolizin|thiophen|thiolo|imidazolidin|imidazol|pyrimidin|pyridin|" + \
                    "piperidin|morpholin|pyrazol|pyridazin|oxocinnolin|cinnolin|pyrrol|thiochromen|oxochromen|chromen|quinazolin|phthalazin|quinoxalin|carbazol|xanthen|pyrazin|purin|" + \
                    "indazol|naphthyridin|quinolizin|guanidin|pyranthren|pyran|thianthren|thian|acridin|acrido|yohimban|porphyrin|pteridin|tetramin|pentamin|" + \
                    "borinin|borino|boriran|borolan|borol|borinan|phenanthridin|quinolin|perimidin|corrin|phenanthrolin|phosphinolin|indacen|silonin|borepin|"

    prefix_heterocycles = "thiaz|oxaza|oxaz|oxan|oxa|ox|aza|az|thia|thioc|thion|thio|thi|telluro|phospha|phosph|selen|bor|sil|alum|ars|germ|tellur|imid|"

    suffix_heterocycles = "ir|et|olo|ol|ino|in|ep|oc|on|ec|"
    saturated_unsatured = "idine|idene|idin|ane|an|ine|in|id|e|"
    pristavka_exception = "do|trisodium|tris|triacetyl|triamine|triaza|triaz|tria|trityl|tri|o"

    type_ = "acid|ether|"
    element = "hydrogen|helium|lithium|beryllium|nitrogen|oxygen|fluorine|neon|sodium|magnesium|aluminum|silicon|" + \
              "phosphorus|sulfur|chlorine|argon|potassium|calcium|scandium|titanium|vanadium|chromium|manganese|iron|" + \
              "cobalt|nickel|copper|zinc|gallium|germanium|arsenic|selenium|bromine|krypton|rubidium|yttrium|zirconium|" + \
              "niobium|molybdenum|technetium|ruthenium|rhodium|palladium|silver|cadmium|indium|antimony|tellurium|iodine|" + \
              "xenon|cesium|barium|lanthanum|cerium|praseodymium|neodymium|latinum|promethium|samarium|europium|gadolinium|" + \
              "terbium|dysprosium|holmium|erbium|thulium|ytterbium|lutetium|hafnium|tantalum|tungsten|rhenium|osmium|" + \
              "iridium|platinum|gold|aurum|mercury|thallium|lead|bismuth|polonium|astatine|radon|francium|radium|actinium|" + \
              "thorium|protactinium|uranium|neptunium|plutonium|americium|curium|berkelium|einsteinium|fermium|californium|" + \
              "mendelevium|nobelium|lawrencium|rutherfordium|dubnium|seaborgium|bohrium|hassium|meitnerium|tin|"

    other_ions = "perchlorate|perbromate|periodate|hypofluorite|hypochlorite|hypobromite|hypoiodite|nitrate|silicate|hydride|"

    regex = re.compile(pattern + heterocycles + carbon_trivial + type_ + element + prefix_isotope + other_ions + alcane + pristavka_digit + pristavka_name + prefix_alcane + \
                       carbon + silicon + prefix_nitrogen + prefix_sulfur + prefix_carbon + prefix_metal + prefix_non_metal + prefix_all + prefix_galogen + prefix_сhalcogen + \
                       suffix_carbon + suffix_nitrogen + suffix_sulfur + suffix_galogen + suffix_сhalcogen + suffix_metal + suffix_non_metal + suffix_all + suffix_heterocycles + \
                       suffix_isotope + boron + selenium  + prefix_heterocycles + saturated_unsatured + pristavka_exception)
    tokens = [token for token in regex.findall(iupac)]

    split_tokens(['meth', 'ane'], tokens)
    split_tokens(['meth', 'an'], tokens)
    split_tokens(['eth', 'ane'], tokens)
    split_tokens(['eth', 'an'], tokens)
    split_tokens(['prop', 'ane'], tokens)
    split_tokens(['prop', 'an'], tokens)
    split_tokens(['but', 'ane'], tokens)
    split_tokens(['but', 'an'], tokens)
    split_tokens(['pent', 'ane'], tokens)
    split_tokens(['pent', 'an'], tokens)
    split_tokens(['hex', 'ane'], tokens)
    split_tokens(['hex', 'an'], tokens)
    split_tokens(['hept', 'ane'], tokens)
    split_tokens(['hept', 'an'], tokens)
    split_tokens(['oct', 'ane'], tokens)
    split_tokens(['oct', 'an'], tokens)
    split_tokens(['non', 'ane'], tokens)
    split_tokens(['non', 'an'], tokens)
    split_tokens(['dec', 'ane'], tokens)
    split_tokens(['dec', 'an'], tokens)
    split_tokens(['cos', 'ane'], tokens)
    split_tokens(['cos', 'an'], tokens)
    split_tokens(['cont', 'ane'], tokens)
    split_tokens(['cont', 'an'], tokens)
    split_tokens(['icos', 'ane'], tokens)
    split_tokens(['icos', 'an'], tokens)

    split_tokens(['thi', 'az'], tokens)
    split_tokens(['thi', 'oc'], tokens)
    split_tokens(['thi', 'on'], tokens)
    split_tokens(['benz', 'ox'], tokens)
    split_tokens(['benz', 'oxa'], tokens)
    split_tokens(['benz', 'ox', 'az'], tokens)
    split_tokens(['benz', 'ox', 'aza'], tokens)
    split_tokens(['phen', 'ox'], tokens)
    split_tokens(['phen', 'oxy'], tokens)
    split_tokens(['phen', 'oxa'], tokens)
    split_tokens(['phen', 'ox', 'az'], tokens)
    split_tokens(['phen', 'ox', 'aza'], tokens)
    split_tokens(['phen', 'ol'], tokens)
    split_tokens(['en', 'oxy'], tokens)
    split_tokens(['ox', 'az'], tokens)
    split_tokens(['ox', 'aza'], tokens)
    split_tokens(['tri', 'az'], tokens)
    split_tokens(['tri', 'amine'], tokens)
    split_tokens(['tri', 'acetyl'], tokens)
    split_tokens(['ox', 'ol'], tokens)
    split_tokens(['ox', 'olo'], tokens)
    split_tokens(['ox', 'an'], tokens)
    split_tokens(['ox', 'oc'], tokens)
    split_tokens(['ox', 'on'], tokens)
    split_tokens(['tri', 'az'], tokens)
    split_tokens(['tri', 'aza'], tokens)
    split_tokens(['tri', 'sodium'], tokens)
    split_tokens(['tetr', 'az'], tokens)
    split_tokens(['tetr', 'aza'], tokens)
    split_tokens(['pent', 'az'], tokens)
    split_tokens(['pent', 'aza'], tokens)
    split_tokens(['hex', 'aza'], tokens)
    split_tokens(['hept', 'aza'], tokens)
    split_tokens(['oct', 'aza'], tokens)
    split_tokens(['non', 'aza'], tokens)
    split_tokens(['dec', 'aza'], tokens)
    split_tokens(['oxo', 'chromen'], tokens)
    split_tokens(['oxo', 'cinnolin'], tokens)
    split_tokens(['oxo', 'cyclo'], tokens)
    split_tokens(['thio', 'chromen'], tokens)
    split_tokens(['thio', 'cyanato'], tokens)

    if (len(re.findall(re.compile('[0-9]{2}\-[a-z]\]'), ''.join(tokens))) > 0):
        for i, tok in enumerate(tokens):
            if re.findall(re.compile('[0-9]{2}\-[a-z]\]'), tok):
                tokens[i] = tok[:2]
                tokens.insert(i+1,tok[2])
                tokens.insert(i+2,tok[3])
                tokens.insert(i+3,tok[4])

    if (len(re.findall(re.compile('[0-9]\-[a-z]\]'), ''.join(tokens))) > 0):
        for i, tok in enumerate(tokens):
            if re.findall(re.compile('[0-9]\-[a-z]\]'), tok):
                tokens[i] = tok[:1]
                tokens.insert(i+1,tok[1])
                tokens.insert(i+2,tok[2])
                tokens.insert(i+3,tok[3])

    if (len(re.findall(re.compile('\[[a-z]{2}\]'), ''.join(tokens))) > 0):
        for i, tok in enumerate(tokens):
            if re.findall(re.compile('\[[a-z]{2}\]'), tok):
                tokens[i] = tok[0]
                tokens.insert(i+1,tok[1])
                tokens.insert(i+2,tok[2])
                tokens.insert(i+3,tok[3])

    if (len(re.findall(re.compile('\[[a-z]\]'), ''.join(tokens))) > 0):
        for i, tok in enumerate(tokens):
            if re.findall(re.compile('\[[a-z]\]'), tok):
                tokens[i] = tok[0]
                tokens.insert(i+1,tok[1])
                tokens.insert(i+2,tok[2])

    if (len(re.findall(re.compile("[0-9]{2}'[a-z]"), ''.join(tokens))) > 0):
        for i, tok in enumerate(tokens):
            if re.findall(re.compile("[0-9]{2}'[a-z]"), tok):
                tokens[i] = tok[:2]
                tokens.insert(i+1,tok[2])
                tokens.insert(i+2,tok[3])

    if (len(re.findall(re.compile("[0-9]'[a-z]"), ''.join(tokens))) > 0):
        for i, tok in enumerate(tokens):
            if re.findall(re.compile("[0-9]'[a-z]"), tok):
                tokens[i] = tok[0]
                tokens.insert(i+1,tok[1])
                tokens.insert(i+2,tok[2])

    if (len(re.findall(re.compile("[0-9]{2}[a-z]"), ''.join(tokens))) > 0):
        for i, tok in enumerate(tokens):
            if re.findall(re.compile("[0-9]{2}[a-z]"), tok):
                tokens[i] = tok[:2]
                tokens.insert(i+1,tok[2])

    if (len(re.findall(re.compile("[0-9][a-z]"), ''.join(tokens))) > 0):
        for i, tok in enumerate(tokens):
            if re.findall(re.compile("[0-9][a-z]"), tok):
                tokens[i] = tok[0]
                tokens.insert(i+1,tok[1])

    if (len(re.findall(re.compile("\.\d+,\d+"), ''.join(tokens))) > 0):
        for i, tok in enumerate(tokens):
            if re.findall(re.compile("\.\d+,\d+"), tok):
                result = add_symbol(tok)
                tokens[i] = result[0]
                for k, res in enumerate(result[1:], start=1):
                    tokens.insert(i+k,res)

    if check_correct_tokenize(iupac, tokens) == True:
        return tokens
    else:
        return None
    


class Iupac_tokenizer(object):
    def __init__(self):
        self.special_tokens =  ['[PAD]','[CLS]','[EOS]','[unk]',';', '.', '>']
        vocab = {'': 7, ' ': 8, "'": 9, '(': 10, ')': 11, ',': 12, '-': 13, '0': 14, '1': 15, '10': 16, '11': 17, '12': 18, '13': 19, '14': 20, '15': 21, '16': 22, '17': 23, '18': 24, '19': 25, '2': 26, '20': 27, '21': 28, '22': 29, '23': 30, '24': 31, '25': 32, '26': 33, '27': 34, '28': 35, '29': 36, '3': 37, '31': 38, '4': 39, '48': 40, '5': 41, '58': 42, '6': 43, '7': 44, '8': 45, '9': 46, 'C': 47, 'E': 48, 'H': 49, 'N': 50, 'O': 51, 'R': 52, 'S': 53, 'Z': 54, '[': 55, ']': 56, '^': 57, 'a': 58, 'acenaphthylen': 59, 'acet': 60, 'acetaldehyde': 61, 'aceto': 62, 'acetyl': 63, 'acid': 64, 'acridin': 65, 'adamant': 66, 'al': 67, 'amide': 68, 'amido': 69, 'amine': 70, 'amino': 71, 'an': 72, 'ane': 73, 'aniline': 74, 'anilino': 75, 'annulen': 76, 'anthracen': 77, 'ate': 78, 'az': 79, 'aza': 80, 'azanida': 81, 'azanide': 82, 'azanidyl': 83, 'azanium': 84, 'azido': 85, 'azonia': 86, 'azonio': 87, 'azulen': 88, 'b': 89, 'benz': 90, 'benzaldehyde': 91, 'benzhydryl': 92, 'benzo': 93, 'benzyl': 94, 'bi': 95, 'bis': 96, 'bromide': 97, 'bromo': 98, 'but': 99, 'buta': 100, 'butanoyl': 101, 'butyl': 102, 'c': 103, 'carbaldehyde': 104, 'carbamate': 105, 'carbamic': 106, 'carbamimidate': 107, 'carbamimidothioate': 108, 'carbamimidoyl': 109, 'carbamo': 110, 'carbamoyl': 111, 'carbazol': 112, 'carbo': 113, 'carbodithioate': 114, 'carbohydrazide': 115, 'carbohydrazonate': 116, 'carbonate': 117, 'carbonimidoyl': 118, 'carbonitrile': 119, 'carbono': 120, 'carbonochloridate': 121, 'carbonyl': 122, 'carbothialdehyde': 123, 'carbothioate': 124, 'carbothioic': 125, 'carbothioyl': 126, 'carboxamide': 127, 'carboximidamide': 128, 'carboximidate': 129, 'carboximidothioate': 130, 'carboximidoyl': 131, 'carboxy': 132, 'carboxylate': 133, 'carboxylato': 134, 'carboxylic': 135, 'chloride': 136, 'chloridoyl': 137, 'chloro': 138, 'chromen': 139, 'chrysen': 140, 'cinnolin': 141, 'conta': 142, 'cos': 143, 'cosa': 144, 'cuban': 145, 'cyanamide': 146, 'cyanato': 147, 'cyanide': 148, 'cyano': 149, 'cyclo': 150, 'd': 151, 'dec': 152, 'deca': 153, 'di': 154, 'do': 155, 'e': 156, 'ec': 157, 'en': 158, 'ene': 159, 'eno': 160, 'ep': 161, 'et': 162, 'eth': 163, 'ethyl': 164, 'f': 165, 'fluoranthen': 166, 'fluoren': 167, 'fluoride': 168, 'fluoro': 169, 'formamide': 170, 'formamido': 171, 'formate': 172, 'formic': 173, 'formyl': 174, 'furan': 175, 'furo': 176, 'g': 177, 'guanidin': 178, 'h': 179, 'heni': 180, 'hept': 181, 'hepta': 182, 'heptalen': 183, 'heptanoyl': 184, 'heptyl': 185, 'hex': 186, 'hexa': 187, 'hexanoyl': 188, 'hexyl': 189, 'hydrazide': 190, 'hydrazin': 191, 'hydrazine': 192, 'hydrazonate': 193, 'hydrazono': 194, 'hydro': 195, 'hydroxy': 196, 'hydroxyl': 197, 'hypofluorite': 198, 'ic': 199, 'icosa': 200, 'id': 201, 'idene': 202, 'idin': 203, 'idine': 204, 'imid': 205, 'imidazo': 206, 'imidazol': 207, 'imidazolidin': 208, 'imido': 209, 'imine': 210, 'imino': 211, 'in': 212, 'indazol': 213, 'inden': 214, 'indol': 215, 'indolizin': 216, 'ino': 217, 'iodo': 218, 'ir': 219, 'iso': 220, 'ium': 221, 'meth': 222, 'methyl': 223, 'methylidene': 224, 'morpholin': 225, 'naphthalen': 226, 'naphtho': 227, 'naphthyridin': 228, 'nitrate': 229, 'nitrile': 230, 'nitro': 231, 'nitroso': 232, 'nitrous': 233, 'non': 234, 'nona': 235, 'o': 236, 'oate': 237, 'oc': 238, 'oct': 239, 'octa': 240, 'octyl': 241, 'oic': 242, 'ol': 243, 'olo': 244, 'on': 245, 'one': 246, 'ox': 247, 'oxa': 248, 'oxamide': 249, 'oxide': 250, 'oxido': 251, 'oxino': 252, 'oxo': 253, 'oxy': 254, 'pent': 255, 'penta': 256, 'pentalen': 257, 'pentanoyl': 258, 'pentyl': 259, 'perimidin': 260, 'peroxy': 261, 'phen': 262, 'phenacyl': 263, 'phenanthren': 264, 'phenanthridin': 265, 'phenanthro': 266, 'phenanthrolin': 267, 'pheno': 268, 'phenyl': 269, 'phosph': 270, 'phosphane': 271, 'phosphanium': 272, 'phosphanyl': 273, 'phosphate': 274, 'phosphonato': 275, 'phosphonia': 276, 'phosphoryl': 277, 'phthalate': 278, 'phthalazin': 279, 'piperazin': 280, 'piperidin': 281, 'prop': 282, 'propa': 283, 'propanoyl': 284, 'propyl': 285, 'pteridin': 286, 'purin': 287, 'pyran': 288, 'pyrazin': 289, 'pyrazol': 290, 'pyren': 291, 'pyridazin': 292, 'pyridin': 293, 'pyrido': 294, 'pyrimidin': 295, 'pyrimido': 296, 'pyrrol': 297, 'pyrrolidin': 298, 'pyrrolizin': 299, 'quinazolin': 300, 'quinolin': 301, 'quinolizin': 302, 'quinoxalin': 303, 'spiro': 304, 'sulfamate': 305, 'sulfamoyl': 306, 'sulfanium': 307, 'sulfanyl': 308, 'sulfanylidene': 309, 'sulfate': 310, 'sulfido': 311, 'sulfinamide': 312, 'sulfinic': 313, 'sulfino': 314, 'sulfinyl': 315, 'sulfonamide': 316, 'sulfonamido': 317, 'sulfonate': 318, 'sulfonato': 319, 'sulfonimidoyl': 320, 'sulfono': 321, 'sulfonyl': 322, 'terephthalate': 323, 'tert-butyl': 324, 'tetr': 325, 'tetra': 326, 'tetracen': 327, 'thi': 328, 'thia': 329, 'thian': 330, 'thio': 331, 'thiocyanate': 332, 'thiol': 333, 'thiolo': 334, 'thione': 335, 'thiophen': 336, 'thiourea': 337, 'tri': 338, 'tria': 339, 'tris': 340, 'trityl': 341, 'un': 342, 'urea': 343, 'xanthen': 344, 'yl': 345, 'yn': 346, 'yne': 347, '[PAD]': 0, '[CLS]': 1, '[EOS]': 2, '[unk]': 3, ';': 4, '.': 5, '>': 6}
        # if vocab is None:
        #     names = [iupac_tokenizer(name) for name in names]
        #     for i in range(len(names)):
        #         tokens = names[i]
        #         if tokens is not None:
        #             tokens = set(tokens)
        #             tokens -= set(self.special_tokens)
        #             vocab |= tokens
        #     nspec = len(self.special_tokens)
        #     self.vocab = dict(zip(sorted(vocab),
        #                         range(nspec, nspec+len(vocab))))
        # else:
        self.vocab = vocab
        # for i,spec in enumerate(self.special_tokens):
        #     self.vocab[spec] = i
        self.rev_vocab = dict((v,k) for k,v in self.vocab.items())
        self.vocsize = len(self.vocab)        

    def tokenize(self,iupac_name):
        tokens = iupac_tokenizer(iupac_name)
        if tokens is None:
            raise Exception(f"Unable to tokenize IUPAC name: {iupac_name}")
        else:
            return tokens
    
    def convert_tokens_to_ids(self, tokens):
        
        return [self.vocab.get(t, self.vocab['[unk]']) for t in tokens]

    def convert_ids_to_tokens(self,ids):
        
        return [self.rev_vocab.get(i, '[unk]') for i in ids]

"""
selfies tokenizer
"""

import selfies as sf
def selfies_tokenizer(sel):
    word = []
    i = 0
    while i<len(sel):
        word_space1 = []
        if sel[i]=="[":
            word_space1.append(sel[i])
            i=i+1
            while sel[i]!="]":
                word_space1.append(sel[i])
                i=i+1
            word_space1.append(sel[i])
            word_space2=''.join(word_space1)
            word.append(word_space2)
            i=i+1   
        else:
            word_space1.append(sel[i])
            word.append(word_space1)
            i=i+1  
            
    nums_list = [ s for s in word]
    return nums_list

class Selfies_tokenizer(object):
    def __init__(self):
        self.vocab =  {'[PAD]': 0, '[CLS]': 1, '[MASK]': 2, '[EOS]': 3, '[unk]': 4, '[#Branch1]': 5, '[#Branch2]': 6, '[#C]': 7, '[#N+1]': 8, '[#N]': 9, '[-/Ring1]': 10, '[-/Ring2]': 11, '[-\\Ring1]': 12, '[/Br]': 13, '[/C@@H1]': 14, '[/C@@]': 15, '[/C@H1]': 16, '[/C@]': 17, '[/C]': 18, '[/Cl]': 19, '[/F]': 20, '[/N+1]': 21, '[/N-1]': 22, '[/NH1+1]': 23, '[/NH1-1]': 24, '[/NH1]': 25, '[/NH2+1]': 26, '[/N]': 27, '[/O+1]': 28, '[/O-1]': 29, '[/O]': 30, '[/S-1]': 31, '[/S@]': 32, '[/S]': 33, '[=Branch1]': 34, '[=Branch2]': 35, '[=C]': 36, '[=N+1]': 37, '[=N-1]': 38, '[=NH1+1]': 39, '[=NH2+1]': 40, '[=N]': 41, '[=O+1]': 42, '[=OH1+1]': 43, '[=O]': 44, '[=P@@]': 45, '[=P@]': 46, '[=PH2]': 47, '[=P]': 48, '[=Ring1]': 49, '[=Ring2]': 50, '[=S+1]': 51, '[=S@@]': 52, '[=S@]': 53, '[=SH1+1]': 54, '[=S]': 55, '[Br]': 56, '[Branch1]': 57, '[Branch2]': 58, '[C@@H1]': 59, '[C@@]': 60, '[C@H1]': 61, '[C@]': 62, '[CH1-1]': 63, '[CH2-1]': 64, '[C]': 65, '[Cl]': 66, '[F]': 67, '[I]': 68, '[N+1]': 69, '[N-1]': 70, '[NH1+1]': 71, '[NH1-1]': 72, '[NH1]': 73, '[NH2+1]': 74, '[NH3+1]': 75, '[N]': 76, '[O-1]': 77, '[O]': 78, '[P+1]': 79, '[P@@H1]': 80, '[P@@]': 81, '[P@]': 82, '[PH1+1]': 83, '[PH1]': 84, '[P]': 85, '[Ring1]': 86, '[Ring2]': 87, '[S+1]': 88, '[S-1]': 89, '[S@@+1]': 90, '[S@@]': 91, '[S@]': 92, '[S]': 93, '[\\Br]': 94, '[\\C@@H1]': 95, '[\\C@H1]': 96, '[\\C]': 97, '[\\Cl]': 98, '[\\F]': 99, '[\\I]': 100, '[\\N+1]': 101, '[\\N-1]': 102, '[\\NH1+1]': 103, '[\\NH1]': 104, '[\\NH2+1]': 105, '[\\N]': 106, '[\\O-1]': 107, '[\\O]': 108, '[\\S-1]': 109, '[\\S@]': 110, '[\\S]': 111}
        self.rev_vocab = dict((v,k) for k,v in self.vocab.items())
        self.vocsize = len(self.vocab)        

    def tokenize(self,sfs_name):
        tokens = selfies_tokenizer(sfs_name)
        if tokens is None:
            raise Exception(f"Unable to tokenize IUPAC name: {sfs_name}")
        else:
            return tokens
    
    def convert_tokens_to_ids(self, tokens):
        num_list = []
        keys = list(self.vocab.keys())
        for t in tokens:
            if t not in keys:
                num_list.append(4)
            else:
                num_list.append(self.vocab.get(t))
        return num_list
    
        # num_list =  [self.vocab.get(t) if self.vocab.get(t) else 4 for t in tokens]

    def convert_ids_to_tokens(self,ids):
        
        return [self.rev_vocab.get(i, '[unk]') for i in ids]


import torch
from torch import nn
from model.TMMF import TMMF
from dataset import create_dataset,create_dataloader
import ruamel.yaml as yaml
import torch.nn.functional as F
import numpy as np
import argparse
from pathlib import Path

class Config(dict):
    def __init__(self, *args, **kwargs):
        super(Config, self).__init__(*args, **kwargs)

    def __getattr__(self, key):
        value = self[key]
        if isinstance(value, dict):
            value = Config(value)
        return value
    


# calculate topk recall rate

def recall_topk(model,loader,device,topk):
    recall_s2i = []
    recall_i2s = []
    recall_s2m = []
    recall_m2s = []
    model.to(device)
    model.eval()
    with torch.no_grad():
        for batch in loader:
            ids,selfies,iupac,mol = batch['ids'],batch['sfs_tokens']['selfies_tokens'],batch['ipc_tokens']['iupac_tokens'],batch['mlg']
            selfies = selfies.to(device)
            iupac = iupac.to(device)
            mol = mol.to(device)  
            
            iupac_embeds,_ = model.iupac_encoder(input_ids=iupac)
            iupac_feat = F.normalize(model.iupac_proj(iupac_embeds[:,0,:]), dim=-1)

            selfies_embeds,_ = model.selfies_encoder(input_ids = selfies)
            selfies_feat = F.normalize(model.selfies_proj(selfies_embeds[:,0,:]), dim=-1)

            mol_pool,_,_ = model.mol_encoder(mol)
            mol_feat = F.normalize(model.mol_proj(mol_pool), dim=-1)

            sim_s2i = selfies_feat @ iupac_feat.t() / model.temp
            sim_i2s = iupac_feat @ selfies_feat.t() / model.temp

            sim_s2m = selfies_feat @ mol_feat.t() / model.temp
            sim_m2s = mol_feat @ selfies_feat.t() / model.temp

            sim_s2i = F.softmax(sim_s2i, dim=-1)
            sim_i2s = F.softmax(sim_i2s, dim=-1)
            sim_s2m = F.softmax(sim_s2m, dim=-1)
            sim_m2s = F.softmax(sim_m2s, dim=-1)

            # 通过sim矩阵计算当前bacth中的topk recall rate
            # sim矩阵的每一行代表一个selfies的相似度分布, 对角线上的为正例，取相似度最高的前k个，如果正确的正例在其中，那么recall+1

            # sim_s2i 代码如下
            sim_s2i = sim_s2i.cpu().numpy()
            for i in range(sim_s2i.shape[0]):
                topk_idx = np.argsort(sim_s2i[i,:])[::-1][:topk]
                if i in topk_idx:
                    recall_s2i.append(1)
                else:
                    recall_s2i.append(0)
            # sim_i2s
            sim_i2s = sim_i2s.cpu().numpy()
            for i in range(sim_i2s.shape[0]):
                topk_idx = np.argsort(sim_i2s[i,:])[::-1][:topk]
                if i in topk_idx:
                    recall_i2s.append(1)
                else:
                    recall_i2s.append(0)
            # sim_s2m
            sim_s2m = sim_s2m.cpu().numpy()
            for i in range(sim_s2m.shape[0]):
                topk_idx = np.argsort(sim_s2m[i,:])[::-1][:topk]
                if i in topk_idx:
                    recall_s2m.append(1)
                else:
                    recall_s2m.append(0)
            # sim_m2s
            sim_m2s = sim_m2s.cpu().numpy()
            for i in range(sim_m2s.shape[0]):
                topk_idx = np.argsort(sim_m2s[i,:])[::-1][:topk]
                if i in topk_idx:
                    recall_m2s.append(1)
                else:
                    recall_m2s.append(0)


    recall_rate_s2i = sum(recall_s2i)/len(recall_s2i)
    recall_rate_i2s = sum(recall_i2s)/len(recall_i2s)
    recall_rate_s2m = sum(recall_s2m)/len(recall_s2m)
    recall_rate_m2s = sum(recall_m2s)/len(recall_m2s)

    return recall_rate_s2i,recall_rate_i2s,recall_rate_s2m,recall_rate_m2s


# zero-shot retrieval

def main(args,config):
    device = args.device
    topk = args.topk
    output_dir = args.output_dir

    print("creating dataset")
    dataset,_ = create_dataset(
    config['dataset']['ids_path'],
    config['dataset']['ipc_path'],
    config['dataset']['sfs_path'],
    config['dataset']['mlg_path'],
    1.0)

    loader = create_dataloader(dataset,1024)
    model = TMMF(config)
    model = model.to(device)

    path = './pre_train/1m_checkpoint_09.pth'
    checkpoint = torch.load(path)
    model.load_state_dict(checkpoint['model'])

    recall_rate_s2i,recall_rate_i2s,recall_rate_s2m,recall_rate_m2s = recall_topk(model,loader,device,topk)
    print("recall_rate_s2i: ",recall_rate_s2i)
    print("recall_rate_i2s: ",recall_rate_i2s)
    print("recall_rate_s2m: ",recall_rate_s2m)
    print("recall_rate_m2s: ",recall_rate_m2s)

    with open(output_dir+ 'top{}'.format(topk) +'_recall_rate.txt','w') as f:
        f.write("recall_rate_s2i: "+str(recall_rate_s2i)+'\n')
        f.write("recall_rate_i2s: "+str(recall_rate_i2s)+'\n')
        f.write("recall_rate_s2m: "+str(recall_rate_s2m)+'\n')
        f.write("recall_rate_m2s: "+str(recall_rate_m2s)+'\n')


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--config', default='./config/retrieval.yaml')
    parser.add_argument('--output_dir', default='./downstream/retrieval/')
    parser.add_argument('--device', default=torch.device("cuda:0"))
    parser.add_argument('--topk', default=1)
    args = parser.parse_args()

    config = yaml.load(open(args.config, 'r'), Loader=yaml.Loader)

    Path(args.output_dir).mkdir(parents=True, exist_ok=True)
     
    main(args, config)








from craft_mol.batch_feature import BatchFeature

model_path = './craft_mol/1m_checkpoint_09.pth'
batch_size = 32
config_path = './craft_mol/inference.yaml'
batch_feature = BatchFeature(model_path=model_path, config_path=config_path)
smiles = "CCO"
iupac_name = "ethanol"
mol_feature,mlg_ids, ipc_ids, sfs_ids = batch_feature.get_feature_single(smiles=smiles, iupac_name=iupac_name)
print(mlg_ids[0].x)



import json
import pubchempy
import selfies as sf
from concurrent.futures import ThreadPoolExecutor, as_completed
from tqdm import tqdm
import time
import os

# 加载自定义数据集，区分 train 和 test 数据
def load_custom_dataset(json_file_path):
    """加载自定义数据集，区分 train 和 test 数据"""
    train_data = []
    test_data = []

    # 读取 JSON 文件
    with open(json_file_path, "r", encoding="utf-8") as f:
        data = json.load(f)

    # 遍历数据，按照 'split' 将数据分配到 train 或 test
    for item in data:
        split = item["metadata"].get("split", "train")  # 默认是 'train'
        if split == "train":
            train_data.append(item)
        elif split == "test":
            test_data.append(item)
        else:
            # 如果有其他的 split 类型，可以根据需要处理
            print(f"Unknown split: {split}")

    return train_data, test_data


# 提取数据中的SELFIES并转换为SMILES
def extract_smiles_from_selfies(selfies_tokens):
    return sf.decoder(selfies_tokens)


# 函数：获取IUPAC名称，添加超时处理
def get_iupac_name(smiles, timeout=60):
    try:
        compounds = pubchempy.get_compounds(smiles, namespace='smiles', timeout=timeout)
        if compounds:
            return compounds[0].iupac_name
        return None
    except Exception as e:
        print(f"Error fetching IUPAC name for {smiles}: {e}")
        return None


# 并行获取IUPAC名称
def fetch_data_for_index(i, dataset):
    sample = dataset[i]
    selfies_tokens = sample["input"]  # 获取SELFIES
    smiles = extract_smiles_from_selfies(selfies_tokens) if selfies_tokens else None  # 转换SELFIES为SMILES
    # 使用索引作为CID
    cid = i
    
    # 获取IUPAC名称
    if smiles:
        iupac_name = get_iupac_name(smiles)
        if iupac_name:
            return {"cid": cid, "smiles": smiles, "iupac_name": iupac_name, "instruction": sample["instruction"], "output": sample["output"], "metadata": sample["metadata"]}
        else:
            return {"cid": cid, "smiles": smiles, "iupac_name": None, "instruction": sample["instruction"], "output": sample["output"], "metadata": sample["metadata"]}
    return {"cid": cid, "smiles": None, "iupac_name": None, "instruction": sample["instruction"], "output": sample["output"], "metadata": sample["metadata"]}


def save_to_json_file(results, output_file, file_exists):
    """将结果保存到 JSON 文件中"""
    if file_exists:
        # 如果文件存在，先读取已有的数据
        with open(output_file, 'r+', encoding='utf-8') as f:
            existing_data = json.load(f)
            existing_data.extend(results)  # 合并新数据和已有数据
            f.seek(0)  # 回到文件开头
            json.dump(existing_data, f, ensure_ascii=False, indent=4)
    else:
        # 如果文件是空的，创建一个新的文件并写入数据
        with open(output_file, 'w', encoding='utf-8') as f:
            json.dump(results, f, ensure_ascii=False, indent=4)
    
    print(f"定时保存了 {len(results)} 条数据到文件")

# 批量爬取IUPAC名称并保存为新的JSON文件
def get_iupac_index_range(start_index, length, dataset, max_len, output_file, failed_indexes_file):
    save_interval=100
    if start_index + length > max_len:
        length = max_len - start_index
    
    # 准备存储数据的列表
    results = []
    failed_indexes = []
    file_exists = False
    if not os.path.exists(os.path.dirname(output_file)):
        os.makedirs(os.path.dirname(output_file))  # 如果文件夹不存在则创建文件夹
    # 使用ThreadPoolExecutor并行获取IUPAC名称
    with ThreadPoolExecutor(max_workers=16) as executor:
        futures = []

        # 提交任务
        for i in range(start_index, start_index + length):
            futures.append(executor.submit(fetch_data_for_index, i, dataset))

        # 处理完成的任务
        for i, future in enumerate(tqdm(as_completed(futures), desc="Processing", total=len(futures), unit="sample")):
            result = future.result()
            if result["iupac_name"] is None:
                failed_indexes.append(result["cid"])
            else:
                results.append({
                    "instruction": result["instruction"],
                    "input": result["smiles"],
                    "iupac_name": result["iupac_name"] if result["iupac_name"] else "null",
                    "output": result["output"],
                    "metadata": result["metadata"]
                })
            if (i + 1) % save_interval == 0 or (i + 1) == len(futures):  # 在每 `save_interval` 个任务后保存
                save_to_json_file(results, output_file, file_exists)
                file_exists = True 

    with open(failed_indexes_file, 'a', encoding='utf-8') as f:
        for failed_idx in failed_indexes:
            f.write(f"{failed_idx}\n")

    print(f"数据已成功保存为 JSON 文件：{output_file}")
    print(f"爬取失败的分子索引已保存到 {failed_indexes_file}！")


if __name__ == "__main__":
    # 示例：加载数据集并并行爬取IUPAC名称
    json_file_path = "/home/hukun/code/trimodal_code/craft_mol/data/Mol-Instructions/data/Molecule-oriented_Instructions/molecular_description_generation.json"  # 替换为你的JSON文件路径
    train_data, test_data = load_custom_dataset(json_file_path)
    start_index = 0  # 起始索引
    length = 1000  # 批量处理的大小
    max_len = len(train_data)  # 数据集的最大长度
    print(f"数据集的最大长度：{max_len}")
    for i in range(start_index, max_len, length):
        output_file = f'/home/hukun/code/trimodal_code/craft_mol/data/Mol-Instructions/data/Molecule-oriented_Instructions/molecular_description_generation/test_m_d_g_{i}_{i+length}.json'
        failed_indexes_file = f'/home/hukun/code/trimodal_code/craft_mol/data/Mol-Instructions/data/Molecule-oriented_Instructions/molecular_description_generation/test_failed_indexes_{i}_{i+length}.txt'
        get_iupac_index_range(i, length, train_data, max_len, output_file,failed_indexes_file)
        time.sleep(120)  # 每批次之间暂停2分钟，避免请求过于频繁
        



from datasets import load_dataset

# 通过 load_dataset 加载数据集（假设你已经定义好了 MyDataset 类）
dataset = load_dataset("/home/hukun/code/trimodal_code/craft_mol/data/Mol-Instructions/Mol-Instructions.py", name="Molecule-oriented Instructions")

# 选择你需要的分割
split = "molecular_description_generation"
data = dataset[split]

# 查看数据
print(data[:5])

# 访问字段
for example in data:
    print(f"Instruction: {example['instruction']}")
    print(f"Input: {example['input']}")
    print(f"Output: {example['output']}")
    print(f"Metadata: {example['metadata']}")
    print("="*50)


import pandas as pd
from tqdm import tqdm
import pubchempy




def get_iupac_name(smiles):
    smiles_valid = []
    names = []
    for i in tqdm(range(len(smiles))):
        smi = smiles[i]
        compounds = pubchempy.get_compounds(smi, namespace='smiles')
        match = compounds[0]
        name = match.iupac_name
        if name != None:
            smiles_valid.append(smi)
            names.append(name)
    return smiles_valid, names


if __name__ == '__main__':
    df = pd.read_csv('./data/test.txt')

    smiles = df['SMILES']
    smiles = smiles.tolist()
    smiles_valid, names = get_iupac_name(smiles)
    df = pd.DataFrame({'SMILES': smiles_valid, 'IUPAC': names})
    df.to_csv('./data/smiles_iupac_test.csv', index=False)


import json
import re

# 设置源文件路径和目标文件路径
start = 0
end = 260000
for i in range(start, end, 20000):

    source_file_path = f"/home/hukun/code/trimodal_code/craft_mol/data/Mol-Instructions/data/Molecule-oriented_Instructions/molecular_description_generation/m_d_g_{i}_{i+20000}.json"  # 替换为实际的源文件路径
    output_file_path = f"/home/hukun/code/trimodal_code/craft_mol/data/Mol-Instructions/data/Molecule-oriented_Instructions/molecular_description_generation/preprocessed_m_d_g_{i}_{i+20000}.txt"  # 替换为目标文件路径

    # 读取源文件

    with open(source_file_path, "r", encoding="utf-8") as file:
        raw_data = file.read()

    # 处理 output 字段中的换行符，合并多行内容为一行
    raw_data = re.sub(r'("output":\s*")([^"]*)(\n)([^"]*")', 
                        lambda m: m.group(1) + m.group(2).replace('\n', ' ') + m.group(4), 
                        raw_data, flags=re.DOTALL)
    # 替换反斜杠 `\` 为双反斜杠 `\\`
    raw_data = raw_data.replace('\\', '\\\\')
    def replace_quotes(text):
        if text:
            text_parts = text.split('"')
            if(len(text_parts) > 1):
                modified_text = re.sub(r'"', '\\"', text)
                text = modified_text
        return text

    # 正则表达式替换：只处理 output 和 iupac_name 之间的内容
    # def replacement_function(m):
    #     # 输出三部分
    #     print("Group 1 (before output content):", m.group(1))
    #     print("Group 2 (content of output):", m.group(2))
        
    #     # 返回替换后的文本
    #     return m.group(1) + replace_quotes(m.group(2)) 

    # # 正则表达式替换：只处理 output 和 iupac_name 之间的内容
    # raw_data = re.sub(r'("output":\s*")(.*?)(?=",\n\s*"iupac_name")', 
    #                     replacement_function, 
    #                     raw_data)

    raw_data = re.sub(r'("output":\s*")(.*?)(?=",\n\s*"iupac_name")', 
                        lambda m: m.group(1) + replace_quotes(m.group(2)), 
                        raw_data)

    with open(output_file_path, "w", encoding="utf-8") as output_file:
        output_file.write(raw_data)

    print(f"处理后的文本文件已保存为 {output_file_path}")




from rdkit import Chem
import pandas as pd
import matplotlib.pyplot as plt
from rdkit.Chem.Draw import rdMolDraw2D
from IPython.display import SVG
import matplotlib.cm as cm
from PIL import Image
import io
import argparse


def add_atom_index(mol,pros):

    for atom in mol.GetAtoms():
        atom.SetProp('atomNote', f'{pros[atom.GetIdx()]:.1f}')

    return mol


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--mol_dir', type=str,default='downstream/interact/')
    parser.add_argument('--model_name', type=str,default="esol")
    parser.add_argument('--mol_name_list', type=str, nargs='+',default=['0','1'])
    args = parser.parse_args()

    mol_dir = args.mol_dir + args.model_name + '/'
    for index in args.mol_name_list:
        smi_file = mol_dir + index  + '/layer2_smi.txt'
        smi = open(smi_file, 'r').readlines()[0].strip()

        # 创建分子对象
        mol = Chem.MolFromSmiles(smi)
        num_atoms = mol.GetNumAtoms()
        # mol = add_atom_index(mol)
        print(f"Number of atoms in the molecule: {num_atoms}")
        # 输出每一个原子以及化学键






        # 获取权重大小，对于8个头，每个头都有一个权重图
        for head in range(8):
            csv_file = mol_dir + index + f'/layer2_head{head}.csv'
            df = pd.read_csv(csv_file)
            # 第一行第一列为index名称，对每一列求和得到数组
            weights = df.sum(axis=0).values[1:].tolist()


            # 创建自定义权重数组，与原子数量相同
            

            # 将weights归一化到0-1
            min_val = min(weights)
            max_val = max(weights)
            custom_weights = [(x - min_val) / (max_val - min_val) for x in weights]

            # 创建highlightAtomColor字典，颜色根据其weights决定
            dict_atom_color = {}
            dict_atom_size = {}
            dict_bond_color = {}
            atom_idx = []
            cmap = cm.get_cmap('Blues')
            for i in range(len(custom_weights)):
                color = cmap(custom_weights[i]*0.8)
                color_ = cmap(custom_weights[(i+1)%len(custom_weights)])
                dict_atom_color[i] = (color[0],color[1],color[2])
                dict_atom_size[i] = 0.3
                atom_idx.append(i)
                # bond 为平均值


            bonds = mol.GetBonds()
            bond_list = []
            for bond in bonds:
                bond_idx = bond.GetIdx()
                bond_list.append(bond_idx)
                begin_atom_idx = bond.GetBeginAtomIdx()
                end_atom_idx = bond.GetEndAtomIdx()
                # dict_bond_color是平均值
                dict_bond_color[bond_idx] = ((dict_atom_color[begin_atom_idx][0]+dict_atom_color[end_atom_idx][0])/2,
                                            (dict_atom_color[begin_atom_idx][1]+dict_atom_color[end_atom_idx][1])/2,
                                            (dict_atom_color[begin_atom_idx][2]+dict_atom_color[end_atom_idx][2])/2)
                


            d2d = rdMolDraw2D.MolDraw2DSVG(450,450)
            d2d.drawOptions().useBWAtomPalette()
            d2d.drawOptions().bondLineWidth= 4
            mol = add_atom_index(mol,custom_weights)
            d2d.DrawMolecule(mol,highlightAtoms=atom_idx,highlightAtomColors=dict_atom_color,highlightAtomRadii=dict_atom_size,
                            highlightBonds=bond_list,
                            highlightBondColors=dict_bond_color)
            d2d.FinishDrawing()
            f = SVG(d2d.GetDrawingText())

            #保存在文件中
            with open(mol_dir + index + f'/layer2_head{head}.svg','w') as svg_file:
                svg_file.write(d2d.GetDrawingText()) 


import json
import torch
from torch.utils.data import Dataset
from rdkit.Chem import AllChem
from torch_geometric.data import Batch
import selfies as sf
from .tokenizer import Iupac_tokenizer, Selfies_tokenizer
from .atom_rep import mol_to_graph_data_obj_simple


class JsonDataset(Dataset):
    def __init__(self, 
                 json_path:str=None, 
                 selfies_max_len:int=130, 
                 iupac_max_len:int=130,
                 CID_name:str='CID',
                 smiles_name:str='input',
                 iupac_name:str='iupac name',
                 selfies_name:str=None):
        """
        Initialize the dataset.

        Parameters:
            json_path (str): JSON file path.
            selfies_max_len (int): Maximum length for SELFIES tokens.
            iupac_max_len (int): Maximum length for IUPAC tokens.
        """
        self.selfies_max_len = selfies_max_len
        self.iupac_max_len = iupac_max_len
        self.CID_name = CID_name
        self.smiles_name = smiles_name
        self.iupac_name = iupac_name
        self.selfies_name = selfies_name


        # Load JSON data
        with open(json_path, 'r', encoding='utf-8') as f:
            self.data = json.load(f)

        # Initialize tokenizers
        self.iupac_tokenizer = Iupac_tokenizer()
        self.selfies_tokenizer = Selfies_tokenizer()

        # Define CLS and EOS tokens
        self.iupac_cls_token = self.iupac_tokenizer.convert_tokens_to_ids(['[CLS]'])[0]
        self.iupac_eos_token = self.iupac_tokenizer.convert_tokens_to_ids(['[EOS]'])[0]
        self.selfies_cls_token = self.selfies_tokenizer.convert_tokens_to_ids(['[CLS]'])[0]
        self.selfies_eos_token = self.selfies_tokenizer.convert_tokens_to_ids(['[EOS]'])[0]

    def __len__(self):
        """Return the number of samples in the dataset."""
        return len(self.data)

    def __getitem__(self, idx):
        """
        Get a data sample by index.

        Returns:
            dict: Processed data sample.
        """
        sample = self.data[idx]
        smiles = sample[self.smiles_name]
        if self.selfies_name is not None:
            selfies = sample[self.selfies_name]
        else:
            selfies = None
            try:
                selfies = sf.encoder(smiles)
            except:
                pass
        iupac_name = sample.get(self.iupac_name, None)
        sample_id = sample[self.CID_name]

        # Initialize the output dictionary
        output = {
            'id': sample_id,
            'selfies_tokens': None,
            'iupac_tokens': None,
            'mol_graph': None
        }

        # Process IUPAC name if available
        if iupac_name:
            output['iupac_tokens'] = self._get_iupac_tokens(iupac_name)

        # Convert SMILES to SELFIES


        # Process SELFIES if available
        if selfies:
            output['selfies_tokens'] = self._get_selfies_tokens(selfies)

        # Process molecular graph from SMILES
        output['mol_graph'] = self._get_mol_graph(smiles)

        return output

    def _get_iupac_tokens(self, text):
        """Convert IUPAC name to tokenized format."""
        try:
            tokens = self.iupac_tokenizer.tokenize(text)
            tokens = self.iupac_tokenizer.convert_tokens_to_ids(tokens)
            tokens = self._get_padded_tokens(tokens, self.iupac_max_len, self.iupac_cls_token, self.iupac_eos_token)
            return tokens
        except:
            return None

    def _get_selfies_tokens(self, text):
        """Convert SELFIES to tokenized format."""
        try:
            tokens = self.selfies_tokenizer.tokenize(text)
            tokens = self.selfies_tokenizer.convert_tokens_to_ids(tokens)
            tokens = self._get_padded_tokens(tokens, self.selfies_max_len, self.selfies_cls_token, self.selfies_eos_token)
            return tokens
        except:
            return None

    def _get_mol_graph(self, smiles):
        """Convert SMILES to a molecular graph."""
        mol = AllChem.MolFromSmiles(smiles)
        return mol_to_graph_data_obj_simple(mol) if mol is not None else None

    def _get_padded_tokens(self, tokens, max_len, cls_token, eos_token):
        """Pad the token sequence to the specified length."""
        tokens = tokens[:max_len]
        tokens = [cls_token] + tokens + [eos_token]
        tokens = torch.tensor(tokens, dtype=torch.long)

        padded_tokens = torch.zeros(max_len + 2, dtype=torch.long)
        padded_tokens[:len(tokens)] = tokens
        return padded_tokens


def JsonDataset_collate(inputs):
    """
    Collate function for JsonDataset.

    Args:
        inputs (list): List of processed samples.

    Returns:
        dict: Batch data.
    """
    id_list = []
    selfies_tokens_list = []
    iupac_tokens_list = []
    mol_graph_list = []

    for sample in inputs:
        if sample['selfies_tokens'] is None or sample['iupac_tokens'] is None or sample['mol_graph'] is None:
            continue  # Skip invalid samples

        id_list.append(sample['id'])
        selfies_tokens_list.append(sample['selfies_tokens'])
        iupac_tokens_list.append(sample['iupac_tokens'])
        mol_graph_list.append(sample['mol_graph'])

    # Collate token sequences
    selfies_tokens_collate = torch.stack(selfies_tokens_list) if selfies_tokens_list else None
    iupac_tokens_collate = torch.stack(iupac_tokens_list) if iupac_tokens_list else None
    mol_graph_collate = Batch.from_data_list(mol_graph_list) if mol_graph_list else None

    batch = {
        'ids': id_list,  # Sample IDs
        'selfies_tokens': selfies_tokens_collate,  # SELFIES token batch
        'iupac_tokens': iupac_tokens_collate,  # IUPAC token batch
        'mol_graph': mol_graph_collate  # Molecular graph batch
    }

    return batch



import json
import os
import re

# 设置目录路径，假设所有json文件都在一个文件夹中
directory = '/home/hukun/code/trimodal_code/craft_mol/data/Mol-Instructions/data/Molecule-oriented_Instructions/molecular_description_generation/'  # 替换为实际文件夹路径

# 创建一个空列表用于存储所有数据
merged_data = []

# 定义命名规则的正则表达式
file_pattern = re.compile(r"test_m_d_g_\d+_\d+\.json")

# 遍历目录中的所有文件
for filename in sorted(os.listdir(directory)):
    if file_pattern.match(filename):  # 只处理符合命名规则的文件
        # 拼接文件路径
        file_path = os.path.join(directory, filename)
        print(file_path)        
        # 读取JSON文件
        with open(file_path, "r") as file:
            raw_data = file.read()


        # 解析 JSON 数据
        data = json.loads(raw_data)

            # 为每条数据加上一个 "id" 字段
        for idx, entry in enumerate(data):
            entry["id"] = len(merged_data) + idx  # 确保 id 唯一且连续
        merged_data.extend(data)

# 保存合并后的数据到一个新的 JSON 文件
with open("/home/hukun/code/trimodal_code/craft_mol/data/Mol-Instructions/data/Molecule-oriented_Instructions/molecular_description_generation/test_data.json", "w") as output_file:
    # 写入开头的 "[" 符号，开始一个 JSON 数组
    output_file.write("[\n")
    
    # 写入合并的数据，每条数据之间添加换行符
    for idx, entry in enumerate(merged_data):
        # 为了格式化，每个字段换行
        json.dump(entry, output_file, indent=4)
        if idx < len(merged_data) - 1:
            output_file.write(",\n")  # 除了最后一项外，每条数据后加逗号
        else:
            output_file.write("\n")  # 最后一项不加逗号
    
    # 写入结尾的 "]" 符号，结束 JSON 数组
    output_file.write("]")

print("所有符合条件的文件已合并并保存为")



# smiles转为mol_id以及selfies 和 iupac表示

import pandas as pd
import selfies as sf
from rdkit import Chem
from rdkit.Chem import AllChem


def check_smiles_validity(smiles):
    try:
        m = Chem.MolFromSmiles(smiles)
        if m:
            return True
        else:
            return False
    except:
        return False

def split_rdkit_mol_obj(mol):
    """
    Split rdkit mol object containing multiple species or one species into a
    list of mol objects or a list containing a single object respectively
    :param mol:
    :return:
    """
    smiles = AllChem.MolToSmiles(mol, isomericSmiles=True)
    smiles_list = smiles.split('.')
    mol_species_list = []
    for s in smiles_list:
        if check_smiles_validity(s):
            mol_species_list.append(AllChem.MolFromSmiles(s))
    return mol_species_list

def get_largest_mol(mol_list):
    """
    Given a list of rdkit mol objects, returns mol object containing the
    largest num of atoms. If multiple containing largest num of atoms,
    picks the first one
    :param mol_list:
    :return:
    """
    num_atoms_list = [len(m.GetAtoms()) for m in mol_list]
    largest_mol_idx = num_atoms_list.index(max(num_atoms_list))
    return mol_list[largest_mol_idx]

def create_standardized_mol_id(smiles):
    """
    :param smiles:
    :return: inchi
    """
    if check_smiles_validity(smiles):
        # remove stereochemistry
        smiles = AllChem.MolToSmiles(AllChem.MolFromSmiles(smiles),
                                     isomericSmiles=False)
        mol = AllChem.MolFromSmiles(smiles)
        if mol != None: # to catch weird issue with O=C1O[al]2oc(=O)c3ccc(cn3)c3ccccc3c3cccc(c3)c3ccccc3c3cc(C(F)(F)F)c(cc3o2)-c2ccccc2-c2cccc(c2)-c2ccccc2-c2cccnc21
            if '.' in smiles: # if multiple species, pick largest molecule
                mol_species_list = split_rdkit_mol_obj(mol)
                largest_mol = get_largest_mol(mol_species_list)
                inchi = AllChem.MolToInchi(largest_mol)
            else:
                inchi = AllChem.MolToInchi(mol)
            return inchi
        else:
            return
    else:
        return

import json

df = pd.read_csv('data/smiles_iupac_30k.csv')
smiles = df['smiles'].tolist()
iupac_names = df['iupac'].tolist()
id_data = []
ipc_data = {}
sfs_data = {}
smi_data = {}


for i in range(len(smiles)):
    try:
        id_ = str(i)
        sf_ = sf.encoder(smiles[i])
        iupac_ = iupac_names[i]
        if id_ is not None and sf_ is not None and iupac_ is not None:
            id_data.append(id_)
            ipc_data[id_] = iupac_names[i]
            sfs_data[id_] = sf.encoder(smiles[i])
            smi_data[id_] = smiles[i]
    except:
        continue


def save_json(save_path,data):
    # assert save_path.split('.')[-1] == 'json'
    f = open(save_path, 'w')
    b = json.dumps(data, indent=2)
    f.write(b)
    f.close()
 


save_json('./data/retrieval/ipc_mapper_train.json',ipc_data)
save_json('./data/retrieval/mol_id_train.json',id_data)
save_json('./data/sfs_mapper_train.json',sfs_data)
save_json('./data/smi_mapper_train.json',smi_data)



import pandas as pd
import selfies as sf
from rdkit import Chem
from rdkit.Chem import AllChem
from dataset.tokenizer import Iupac_tokenizer
import json

iupac_tokenizer = Iupac_tokenizer()
df = pd.read_csv('data/smiles_iupac_30k.csv')
smiles = df['smiles'].tolist()
iupac_names = df['iupac'].tolist()
id_data = []
ipc_data = {}
sfs_data = {}
smi_data = {}


for i in range(len(smiles)):
    try:
        id_ = str(i)
        sf_ = sf.encoder(smiles[i])
        iupac_ = iupac_names[i]
        t = iupac_tokenizer.tokenize(iupac_)
        if id_ is not None and sf_ is not None and iupac_ is not None:
            id_data.append(id_)
            ipc_data[id_] = iupac_names[i]
            sfs_data[id_] = sf.encoder(smiles[i])
            smi_data[id_] = smiles[i]
    except:
        continue


def save_json(save_path,data):
    # assert save_path.split('.')[-1] == 'json'
    f = open(save_path, 'w')
    b = json.dumps(data, indent=2)
    f.write(b)
    f.close()
 


save_json('./data/downstream_tasks/retrieval/ipc_mapper_retrieval.json',ipc_data)
save_json('./data/downstream_tasks/retrieval/mol_id_retrieval.json',id_data)
save_json( './data/downstream_tasks/retrieval/sfs_mapper_retrieval.json',sfs_data)
save_json('./data/downstream_tasks/retrieval/smi_mapper_retrieval.json',smi_data)